package com.org.web.data.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.org.web.data.domain.DataStatisticsItem;
import com.org.web.person.domain.BcPersonSupport;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface DataMapper extends BaseMapper<BcPersonSupport> {

    // 各村供养人数
    @Select("select c.name as item_name, c.value as item_remark,  " +
            "                   (select count(1)   " +
            "                       from bc_person_support a " +
            "                       where a.is_delete = 0  " +
            "                       and a.type = 1 " +
            "                       and IFNULL(a.village, '') != '' " +
            "                       and a.village = c.value) as item_value " +
            "            from sys_data_dictionary_item c  " +
            "            where c.is_delete = 0  " +
            "              and c.dictionary_code = '0040' ")
    List<DataStatisticsItem> listVillageCount();

    // 是否精准扶贫
    @Select("SELECT " +
            "    '是' AS item_name, " +
            "    1 as item_remark, " +
            "    SUM(CASE WHEN if_help_poor = 1 THEN 1 ELSE 0 END) AS item_value " +
            "FROM bc_person_support " +
            "UNION " +
            "SELECT " +
            "    '否' AS item_name, " +
            "    0 as item_remark, " +
            "    SUM(CASE WHEN if_help_poor = 0 THEN 1 ELSE 0 END) AS item_value " +
            "FROM bc_person_support " +
            "where is_delete = 0 " +
            "and type = 1 ")
    List<DataStatisticsItem> listIfHelpPoor();

    // 性别
    @Select("SELECT " +
            "    '男' AS item_name, " +
            "    1 as item_remark, " +
            "    SUM(CASE WHEN sex = 1 THEN 1 ELSE 0 END) AS item_value " +
            "FROM bc_person_support " +
            "UNION " +
            "SELECT " +
            "    '女' AS item_name, " +
            "    0 as item_remark, " +
            "    SUM(CASE WHEN sex = 0 THEN 1 ELSE 0 END) AS item_value " +
            "FROM bc_person_support " +
            "where is_delete = 0 " +
            "and type = 1 ")
    List<DataStatisticsItem> listSex();

    // 护理等级
    @Select("select c.name as item_name, c.value as item_remark,  " +
            "                   (select count(1)   " +
            "                       from bc_person_support a " +
            "                       where a.is_delete = 0  " +
            "                       and a.type = 1 " +
            "                       and IFNULL(a.village, '') != '' " +
            "                       and a.village = c.value) as item_value " +
            "            from sys_data_dictionary_item c  " +
            "            where c.is_delete = 0  " +
            "              and c.dictionary_code = '0040' ")
    List<DataStatisticsItem> listNursingLevel();

    // 年龄区间
    @Select("SELECT " +
            "    age_ranges.age_group as item_name, " +
            "    COALESCE(COUNT(age_table.id_card_no), 0) AS item_value " +
            "FROM ( " +
            "    SELECT '50岁以下' AS age_group, 0 AS min_age, 49 AS max_age " +
            "    UNION SELECT '50-59岁', 50, 59 " +
            "    UNION SELECT '60-69岁', 60, 69 " +
            "    UNION SELECT '70-79岁', 70, 79 " +
            "    UNION SELECT '80-89岁', 80, 89 " +
            "    UNION SELECT '90岁以上', 90, 999 " +
            ") AS age_ranges " +
            "LEFT JOIN ( " +
            "    SELECT " +
            "        id_card_no, " +
            "        TIMESTAMPDIFF(YEAR, STR_TO_DATE(SUBSTRING(id_card_no, 7, 8), '%Y%m%d'), CURDATE()) AS age " +
            "    FROM bc_person_support " +
            "   where is_delete = 0 " +
            "   and type = 1  " +
            ") AS age_table ON age_table.age BETWEEN age_ranges.min_age AND age_ranges.max_age " +
            "GROUP BY age_ranges.age_group ")
    List<DataStatisticsItem> listAgeCount();


    // 是否在园
    @Select("SELECT " +
            "    '是' AS item_name, " +
            "    1 as item_remark, " +
            "    SUM(CASE WHEN is_in_hospital = 1 THEN 1 ELSE 0 END) AS item_value " +
            "FROM bc_person_support " +
            "UNION " +
            "SELECT " +
            "    '否' AS item_name, " +
            "    0 as item_remark, " +
            "    SUM(CASE WHEN is_in_hospital = 0 THEN 1 ELSE 0 END) AS item_value " +
            "FROM bc_person_support " +
            "where is_delete = 0 " +
            "and type = 1 ")
    List<DataStatisticsItem> listInHospital();

    // 房源入住情况
    @Select("SELECT " +
            "    '总房源' AS item_name, " +
            "    0 as item_remark, " +
            "    count(1) AS item_value " +
            "FROM bc_room a " +
            "left join bc_room_bed b on a.id = b.room_id " +
            "where a.is_delete = 0 " +
            "and b.is_delete = 0 " +
            "and a.room_type = 1 " +
            "UNION " +
            "select '入住房源' AS item_name, " +
            "    1 as item_remark, " +
            "    count(1) AS item_value " +
            "from bc_hospital_record a " +
            "where a.is_delete = 0 " +
            "and check_out_time is null")
    List<DataStatisticsItem> listRoomCount();
}
